package com.gic.plug.web.controller.goods;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.curator.shaded.com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.goods.api.dto.GoodsSkuDTO;
import com.gic.goods.api.service.GoodsApiService;
import com.gic.goods.api.util.Constant;
import com.gic.mall.base.api.enums.ErrorCodeEnums;
import com.gic.mall.share.api.dto.shop.ShopDTO;
import com.gic.mall.share.api.service.ShopApiService;
import com.gic.plug.web.vo.goods.GoodsVO;
import com.gic.search.business.api.constant.enums.OperateEnum;
import com.gic.search.business.api.dto.DynamicSearchDTO;
import com.gic.search.business.api.dto.ESResponseQueryBatchDTO;
import com.gic.search.business.api.service.EsBusinessOperaApiService;
import com.gic.search.business.api.utils.QueryConditionAssemblyUtil;
import com.gic.widget.screening.api.service.EsScreeningSearchService;

/**
  * @ClassName: GoodsTagController
  * @Description: 商品标签相关接口
  * @author xub
  * @date 2019/8/29 上午10:03
  */
@RestController
public class GoodsController {

    @Autowired
    private EsBusinessOperaApiService esBusinessOperaApiService;

    @Autowired
    private EsScreeningSearchService esScreeningSearchService;

    @Autowired
    private GoodsApiService goodsApiService;

    @Autowired
    private ShopApiService shopApiService;


    /**
     * @Description: 查询所有（不分页）
     * @author xub
     */
    @GetMapping("list-goods")
    public RestResponse ListGoods(@RequestParam(value = "currentPage", defaultValue = "1") Integer currentPage,
                                 @RequestParam(value = "pageSize", defaultValue = "20") Integer pageSize, Long goodsDomainId,String search,String channelCode) {
        ServiceResponse<ESResponseQueryBatchDTO> serviceResponse = searchES(currentPage,pageSize,goodsDomainId,search,channelCode);
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult());
        }
        return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
    }


    /**
     * 商城组对外提供的组件
     * @param currentPage
     * @param pageSize
     * @param appConfigId
     * @param search
     * @param channelCode
     * @param cordId
     * @return
     */
    @RequestMapping("list-goods-selector")
    public RestResponse listGoodsByChannel(@RequestParam(value = "currentPage", defaultValue = "1") Integer currentPage,
                                           @RequestParam(value = "pageSize", defaultValue = "20") Integer pageSize, Long appConfigId, String search, String channelCode, Long cordId) {
        Integer shopType = null;
        if (Objects.equals(channelCode, Constant.CHANNEL_CODE_MALL)) {
            shopType = 1;
        }
        ServiceResponse<ShopDTO> shopDTOServiceResponse =  shopApiService.getShopByAppletId(appConfigId, 1129, shopType);
        if (shopDTOServiceResponse.getResult() == null) {
            return RestResponse.failure(ErrorCodeEnums.MAll_ERR.getCode(), "不存在店铺");
        }
        Long areaId = shopDTOServiceResponse.getResult().getAreaId();
        ServiceResponse<ESResponseQueryBatchDTO> serviceResponse = searchGoods(currentPage, pageSize, areaId, search, channelCode, cordId);
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult());
        }
        return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
    }


    /**
     * @Description: 查询所有（不分页）
     * @author xub
     */
    @GetMapping("list-sku")
    public RestResponse ListSku(@RequestParam(value = "currentPage", defaultValue = "1") Integer currentPage,
                                 @RequestParam(value = "pageSize", defaultValue = "20") Integer pageSize, Long goodsDomainId,String search,String channelCode) {
        ServiceResponse<ESResponseQueryBatchDTO> serviceResponse = searchES(currentPage,pageSize,goodsDomainId,search ,channelCode);
        if (!serviceResponse.isSuccess()) {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
        //商品信息
        List<JSONObject> list = serviceResponse.getResult().getRes();
        if (CollectionUtils.isEmpty(list)) {
            return RestResponse.success(serviceResponse.getResult());
        }
        //1、获取商品信息
        List<GoodsVO> goodsVOS = Lists.newArrayList();
        for (JSONObject jsonObject : list) {
            GoodsVO goodsVO = JSONObject.toJavaObject(jsonObject, GoodsVO.class);
            goodsVOS.add(goodsVO);
        }
       //2、获取商品ID信息
        List<Long> goodsIds = goodsVOS.stream().mapToLong(GoodsVO::getGoodsId).boxed().collect(Collectors.toList());
        ServiceResponse<List<GoodsSkuDTO>> skuResponse = goodsApiService.listSku(goodsIds, channelCode, Constant.TEST_ENTERPRISE_ID);
        if (!skuResponse.isSuccess()) {
            return RestResponse.failure(skuResponse.getCode(), skuResponse.getMessage());

        }
        //3、获取sku信息 同时存储在map中
        List<GoodsSkuDTO> skuDTOList = skuResponse.getResult();
        if (CollectionUtils.isEmpty(skuDTOList)) {
            return RestResponse.success(serviceResponse.getResult());
        }
        Map<Long, List<GoodsSkuDTO>> goodsIdAndskuDto = skuDTOList.stream().collect(Collectors.groupingBy(GoodsSkuDTO::getGoodsId));
        //4、商品信息中包含是sku信息
        for (GoodsVO goodsVO : goodsVOS) {
            if (goodsIdAndskuDto.containsKey(goodsVO.getGoodsId())) {
                List<GoodsVO.SkuVO> skuVOList = EntityUtil.changeEntityListByOrika(GoodsVO.SkuVO.class,goodsIdAndskuDto.get(goodsVO.getGoodsId()));
                goodsVO.setSkuVOList(skuVOList);
            }
        }
        return RestResponse.success(goodsVOS);
    }

    private ServiceResponse<ESResponseQueryBatchDTO> searchGoods( Integer currentPage,Integer pageSize, Long goodsDomainId,String search,String channelCode, Long cordId) {
        DynamicSearchDTO searchDTO = new DynamicSearchDTO();
        String searchCode = "";
        if (Constant.CHANNEL_CODE_MALL.equals(channelCode)) {
            searchCode = "gic_mall_goods";
        }
        String esName = esScreeningSearchService.getCurrentEsName("1129", searchCode);
        searchDTO.setIndexName(esName);
        searchDTO.setColumnCategoryCode(searchCode);
        searchDTO.setEnterpriseId("1129");
        searchDTO.setType("mapper_type");
        searchDTO.setBegin((currentPage - 1) * pageSize);
        searchDTO.setRecordNumber(pageSize);
        JSONObject jsonObject = QueryConditionAssemblyUtil.createSimpleQueryNode("goodsDomainId", OperateEnum.OPERATE_EQ, goodsDomainId);
        JSONObject status = QueryConditionAssemblyUtil.createSimpleQueryNode("status", OperateEnum.OPERATE_EQ, 2);
        jsonObject = QueryConditionAssemblyUtil.addQueryNodeForSiblin(status, jsonObject, OperateEnum.OPERATE_EQ);
        if (StringUtils.isNotBlank(search)) {
            JSONObject goodsCodeJson = QueryConditionAssemblyUtil.createSimpleQueryNode("goodsCode", OperateEnum.OPERATE_LIKE, search);
            JSONObject goodsNameJson = QueryConditionAssemblyUtil.createSimpleQueryNode("goodsName", OperateEnum.OPERATE_LIKE, search);
            List<JSONObject> jsonArray = new ArrayList<>();
            jsonArray.add(goodsCodeJson);
            jsonArray.add(goodsNameJson);
            JSONObject object = QueryConditionAssemblyUtil.createListQueryNode(jsonArray,OperateEnum.OPERATE_OR);
            QueryConditionAssemblyUtil.addQueryNodeForSiblin(object, jsonObject, OperateEnum.OPERATE_AND);
        }
        if (cordId != null) {
            JSONObject cordJSON = QueryConditionAssemblyUtil.createSimpleQueryNode("cordId", OperateEnum.OPERATE_CONTAIN, search);
            QueryConditionAssemblyUtil.addQueryNodeForSiblin(cordJSON, jsonObject, OperateEnum.OPERATE_AND);
        }
        searchDTO.setSearchJson(jsonObject);
        ServiceResponse<ESResponseQueryBatchDTO> serviceResponse = esBusinessOperaApiService.queryDataBatch(searchDTO, false, null);
        return serviceResponse;
    }


    private ServiceResponse<ESResponseQueryBatchDTO> searchES( Integer currentPage,Integer pageSize, Long goodsDomainId,String search,String channelCode) {
        DynamicSearchDTO searchDTO = new DynamicSearchDTO();
        String esName = esScreeningSearchService.getCurrentEsName("goods-test-123456", "goods-search");
        searchDTO.setIndexName(esName);
        searchDTO.setColumnCategoryCode("goods-search");
        searchDTO.setEnterpriseId("goods-test-123456");
        searchDTO.setType("mapper_type");
        searchDTO.setBegin(currentPage);
        searchDTO.setRecordNumber(pageSize);
        JSONObject goodsCodeJson = QueryConditionAssemblyUtil.createSimpleQueryNode("goodsCode", OperateEnum.OPERATE_LIKE, search);
        JSONObject goodsNameJson = QueryConditionAssemblyUtil.createSimpleQueryNode("goodsName", OperateEnum.OPERATE_LIKE, search);
        JSONObject goodsDomainIdJson = QueryConditionAssemblyUtil.createSimpleQueryNode("channelCodeDomainId", OperateEnum.OPERATE_EQ, channelCode+"-"+goodsDomainId);
        JSONObject nameAndCodeJson = QueryConditionAssemblyUtil.addQueryNodeForSiblin(goodsCodeJson, goodsNameJson, OperateEnum.OPERATE_OR);
        JSONObject json = QueryConditionAssemblyUtil.addQueryNodeForSiblin(nameAndCodeJson, goodsDomainIdJson, OperateEnum.OPERATE_AND);
        searchDTO.setSearchJson(json);
        ServiceResponse<ESResponseQueryBatchDTO> serviceResponse = esBusinessOperaApiService.queryDataBatch(searchDTO, false, null);
        return serviceResponse;
    }
}
